
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "file60.h"

int calc_sec(FILE *);

#define	STARTTRK	35			/* Z[uf[^̐擪gbNԍ */

FILE *fopen(const char *fname, const char *str)
{
	FILE	*rt;
	unsigned char	mode;

	if ( (strlen(fname) != 1) || (fname[0] < '1') || (fname[0] > '5') ) {
		return NULL;	/* ERROR */
	}

	if        (strncmp(str,"rb",3) == 0) {
		mode = FILE_MODE_RB;
	} else if (strncmp(str,"wb",3) == 0) {
		mode = FILE_MODE_WB;
	} else {
		return NULL;	/* ERROR */
	}

	rt = (FILE *) malloc(sizeof(FILE));
	if (rt == NULL) {
		return NULL;	/* ERROR */
	}

	rt->mode		= mode;
	rt->err    		= FILE_ERR_NOERR;
	rt->ptr			= 0;
	rt->st_track	= STARTTRK + (fname[0] - '1');
	rt->st_sector	= 0x01;

	calc_sec(rt);
	if (secrd_set(rt->track, rt->sector) != FILE_ERR_NOERR) {
		rt->err = FILE_ERR_ERR;
	}
	rt->buftrack	= rt->track;
	rt->bufsector	= rt->sector;

	return rt;
}

int fclose(FILE *fp)
{
	int		rt;

	if (fp == NULL) {
		return EOF;		/* pointer ERROR */
	}

	if (secwr_set(fp->buftrack, fp->bufsector) == FILE_ERR_ERR) {
		fp->err = FILE_ERR_ERR;
	}

	if (fp->err == FILE_ERR_NOERR) {
		rt = 0;
	} else {
		rt = EOF;		/* file ERROR */
	}
	free(fp);
	return rt;
}

size_t fwrite(const void *buf, size_t size, size_t nmemb, FILE *fp)
{
	size_t	i,j;
	unsigned char	*buftmp;

	if (fp->mode != FILE_MODE_WB) {
		return 0;
	}
	if (fp->err != FILE_ERR_NOERR) {
		return 0;
	}

	buftmp = (unsigned char *)buf;

	for (j = 0; j < nmemb; j ++) {
		for (i = 0; i < size; i ++) {
			if (calc_sec(fp) == EOF) {
				if (secwr_set(fp->buftrack, fp->bufsector) != FILE_ERR_NOERR) {
					fp->err = FILE_ERR_ERR;
					return j;
				}
				if (secrd_set(fp->track, fp->sector) != FILE_ERR_NOERR) {
					fp->err = FILE_ERR_ERR;
					return j;
				}
				fp->buftrack  = fp->track;
				fp->bufsector = fp->sector;
			}
			secwr_byte(fp->ptr, *buftmp);
			buftmp ++;
			fp->ptr ++;
		}
	}

	return j;
}

size_t fread(void *buf, size_t size, size_t nmemb, FILE *fp)
{
	size_t	i,j;
	unsigned char	*buftmp;

	if (fp->mode != FILE_MODE_RB) {
		return 0;
	}
	if (fp->err != FILE_ERR_NOERR) {
		return 0;
	}

	buftmp = (unsigned char *)buf;

	for (j = 0; j < nmemb; j ++) {
		for (i = 0; i < size; i ++) {
			if (calc_sec(fp) == EOF) {
				if (secrd_set(fp->track, fp->sector) != FILE_ERR_NOERR) {
					fp->err = FILE_ERR_ERR;
					return j;
				}
				fp->buftrack  = fp->track;
				fp->bufsector = fp->sector;
			}
			*buftmp = secrd_byte(fp->ptr);
			buftmp ++;
			fp->ptr ++;
		}
	}

	return j;
}

int calc_sec(FILE *fp)
{
	unsigned char	nowsector, nowtrack;
	int		rt;

	nowsector = ( (fp->ptr & 0x0f00) >>  8) + fp->st_sector;
	nowtrack  = ( (fp->ptr & 0xf000) >> 12) + fp->st_track;

	if (nowsector > 16) {
		nowsector -= 16;
		nowtrack ++;
	}

	fp->track  = nowtrack;
	fp->sector = nowsector;

	if (nowtrack == fp->buftrack && nowsector == fp->bufsector) {
		rt = 0;
	} else {
		rt = EOF;
	}

	return rt;
}

